@@ -35,12 +35,12 @@
 #' @examples
 #' # MODEL 1: Linear regression
 #' fit <- lm(Sepal.Length ~ . , data = iris)
-#' s <- hstats(fit, X = iris[-1])
+#' s <- hstats(fit, X = iris[, -1])
 #' plot(pd_importance(s))
 #' 
 #' # MODEL 2: Multi-response linear regression
-#' fit <- lm(as.matrix(iris[1:2]) ~ Petal.Length + Petal.Width + Species, data = iris)
-#' s <- hstats(fit, X = iris[3:5])
+#' fit <- lm(as.matrix(iris[, 1:2]) ~ Petal.Length + Petal.Width + Species, data = iris)
+#' s <- hstats(fit, X = iris[, 3:5])
 #' plot(pd_importance(s))
 pd_importance <- function(object, ...) {
   UseMethod("pd_importance")
@@ -54,8 +54,9 @@ pd_importance.default <- function(object, ...) {
 
 #' @describeIn pd_importance PD based feature importance from "hstats" object.
 #' @export
-pd_importance.hstats <- function(object, normalize = TRUE, squared = TRUE,
-                                 sort = TRUE, zero = TRUE, ...) {
+pd_importance.hstats <- function(
+    object, normalize = TRUE, squared = TRUE, sort = TRUE, zero = TRUE, ...
+  ) {
   get_hstats_matrix(
     statistic = "pd_importance",
     object = object,
@@ -84,3 +85,4 @@ pd_importance_raw <- function(x) {
   num <- .zap_small(num, eps = x[["eps"]])  # Numeric precision
   list(num = num, denom = x[["mean_f2"]])
 }
+
